@@ -1,5 +1,5 @@
 ## (c) Simon N. Wood (ocat, tw, nb, ziP) & Natalya Pya (scat, beta), 
-## 2013, 2014. Released under GPL2.
+## 2013-2015. Released under GPL2.
 
 ## extended families for mgcv, standard components. 
 ## family - name of family character string
@@ -42,7 +42,7 @@
 ocat <- function(theta=NULL,link="identity",R=NULL) {
 ## extended family object for ordered categorical model.
 ## one of theta and R must be supplied. length(theta) == R-2.
-## weights are ignored.
+## weights are ignored. #! is stuff removed under re-definition of ls as 0
   linktemp <- substitute(link)
   if (!is.character(linktemp)) linktemp <- deparse(linktemp)
   if (linktemp %in% c("identity")) stats <- make.link(linktemp)
@@ -133,11 +133,11 @@ ocat <- function(theta=NULL,link="identity",R=NULL) {
     ##f <- pmax(f1 - f0,.Machine$double.eps)
     f <- Fdiff(al0mu,al1mu)
     ##a1 <- f1^2 - f1;a0 <- f0^2 - f0; a <- a1 -a0
-    al1al0 <- (al1-al0)/2;al0al1 <- (al0-al1)/2
+#!    al1al0 <- (al1-al0)/2;al0al1 <- (al0-al1)/2
     ##g1 <- F(al1al0);g0 <- F(al0al1)
     ##A <- pmax(g1 - g0,.Machine$double.eps)
-    A <- Fdiff(al0al1,al1al0)
-    rsd <- 2*(log(A)-log(f))
+#!    A <- Fdiff(al0al1,al1al0)
+    rsd <- -2*log(f) #! 2*(log(A)-log(f))
     attr(rsd,"sign") <- s
     rsd
   } ## end of dev.resids
@@ -207,10 +207,10 @@ ocat <- function(theta=NULL,link="identity",R=NULL) {
     ## a1 <- f1^2 - f1;a0 <- f0^2 - f0; 
     a <- a1 - a0
     
-    al1al0 <- (al1-al0)/2; al0al1 <- (al0-al1)/2
+    #!al1al0 <- (al1-al0)/2; #! al0al1 <- (al0-al1)/2
     ##g1 <- F(al1al0);g0 <- F(al0al1)
     ##A <- pmax(g1 - g0,.Machine$double.eps)
-    A <- Fdiff(al0al1,al1al0)
+#!    A <- Fdiff(al0al1,al1al0)
     if (level>=0) {
       ## b1 <- f1 - 3 * f1^2 + 2 * f1^3;b0 <- f0 - 3 * f0^2 + 2 * f0^3  
       b1 <- r1$bj;b0 <- r0$bj
@@ -221,10 +221,10 @@ ocat <- function(theta=NULL,link="identity",R=NULL) {
       ##c0 <- -f0 + 7 * f0^2 - 12* f0^3 + 6 * f0^4
       c1 <- r1$cj;c0 <- r0$cj
       c <- c1 - c0
-      R1 <- abcd(al1al0,level-2) 
-      R0 <- abcd(al0al1,level-2)
+#!      R1 <- abcd(al1al0,level-2) 
+#!      R0 <- abcd(al0al1,level-2)
       ## B <- g1^2 - g1 + g0^2 - g0
-      B <- R1$aj + R0$aj
+#!     B <- R1$aj + R0$aj
     }
     if (level>1) {
       ##d1 <- f1 - 15 * f1^2 + 50 * f1^3 - 60 * f1^4 + 24 * f1^5
@@ -232,14 +232,14 @@ ocat <- function(theta=NULL,link="identity",R=NULL) {
       d1 <- r1$dj;d0 <- r0$dj
       d <- d1 - d0
       ##C <- 2 * g1^3 - 3 * g1^2 + g1 - 2 * g0^3 + 3 * g0^2 - g0
-      C <- R1$bj - R0$bj
+#!      C <- R1$bj - R0$bj
     }
 
     oo <- list(D=NULL,Dmu=NULL,Dmu2=NULL,Dth=NULL,Dmuth=NULL,
              Dmu2th=NULL)
     n <- length(y)
     ## deviance...
-    oo$D <- 2*(log(A)-log(f))
+    oo$D <- -2 * log(f) #! 2*(log(A)-log(f))
     if (level >= 0) { ## get derivatives used in coefficient estimation
       oo$Dmu <- -2 * a / f
       a2 <- a^2
@@ -254,7 +254,8 @@ ocat <- function(theta=NULL,link="identity",R=NULL) {
       Dmua1 <- -2 * (a1 * a /f - b1)/f
       Dmu2a0 <- -2* (c0 + (a0*(2*a2/f - b)- 2*b0*a  )/f)/f
       Dmu2a1 <- 2*(c1  + (2*(a1*a2/f - b1*a) - a1*b)/f)/f
-      Da0 <- B/A - 2*a0/f; Da1 <- -B/A + 2 * a1/f 
+      Da0 <- -2*a0/f #! + B/A; 
+      Da1 <-  2 * a1/f #! - B/A 
       ## now transform to derivatives w.r.t. theta...
       oo$Dmu2th <- oo$Dmuth <- oo$Dth <- matrix(0,n,R-2)
       for (k in 1:(R-2)) { 
@@ -297,9 +298,9 @@ ocat <- function(theta=NULL,link="identity",R=NULL) {
       Dmu2a0a1 <- 0 ## (8*a0*b1*a/f^3 + 8*b0*a1*a/f^3 - 12*a0*a1*a/f^4 - 4*b0*b1/f^2 +
                     ## 4*a0*a1*b/f^3 - 2*c0*a1/f^2 - 2*c1*a0/f^2)
 
-      Da0a0 <- 2 * (b0 + a0^2/f)/f + .5 * (C - B^2/A)/A
-      Da1a1 <- -2* (b1 - a1^2/f)/f + .5 * (C - B^2/A)/A
-      Da0a1 <- -2*a0*a1/f2 - .5 * (C - B^2/A)/A
+      Da0a0 <- 2 * (b0 + a0^2/f)/f #! + .5 * (C - B^2/A)/A
+      Da1a1 <- -2* (b1 - a1^2/f)/f #! + .5 * (C - B^2/A)/A
+      Da0a1 <- -2*a0*a1/f2 #! - .5 * (C - B^2/A)/A
 
       ## now transform to derivatives w.r.t. theta...
       n2d <- (R-2)*(R-1)/2
@@ -360,7 +361,9 @@ ocat <- function(theta=NULL,link="identity",R=NULL) {
   } ## end aic
 
   ls <- function(y,w,n,theta,scale) {
-    ## the log saturated likelihood function.
+    ## the log saturated likelihood function. 
+    #! actually only first line used since re-def as 0
+    return(list(ls=0,lsth1=rep(0,R-2),lsth2=matrix(0,R-2,R-2)))
     F <- function(x) {
       h <- ind <- x > 0; h[ind] <- 1/(exp(-x[ind]) + 1)
       x <- exp(x[!ind]); h[!ind] <- (x/(1+x))
@@ -427,6 +430,7 @@ ocat <- function(theta=NULL,link="identity",R=NULL) {
       } 
     }
     list(ls=ls,lsth1=colSums(Dth),lsth2=ls2)
+   
   } ## end of ls
   
   ## initialization is interesting -- needs to be with reference to initial cut-points
@@ -945,7 +949,7 @@ betar <- function (theta = NULL, link = "logit",eps=.Machine$double.eps*100) {
     validmu <- function(mu) all(mu > 0 & mu < 1)
 
     dev.resids <- function(y, mu, wt,theta=NULL) {
-    ## '-'2*loglik instead of deviance in REML/ML expression
+    ## '-2*loglik' instead of deviance in REML/ML expression
       if (is.null(theta)) theta <- get(".Theta")
       theta <- exp(theta) ## note log theta supplied
       muth <- mu*theta
@@ -1380,11 +1384,11 @@ scat <- function (theta = NULL, link = "identity") {
 
 ## zero inflated Poisson (Simon Wood)...
 
-lind <- function(l,th,deriv=0) {
+lind <- function(l,th,deriv=0,k=0) {
 ## evaluate th[1] + exp(th[2])*l and some derivs
   th[2] <- exp(th[2])
-  r <- list(p = th[1] + th[2]*l)
-  r$p.l <- th[2]   ## p_l
+  r <- list(p = th[1] + (k+th[2])*l)
+  r$p.l <- k + th[2]   ## p_l
   r$p.ll <- 0 ## p_ll
   if (deriv) {
     n <- length(l);  
@@ -1454,7 +1458,7 @@ logid <- function(l,th,deriv=0,a=0,trans=TRUE) {
 
 
 
-ziP <- function (theta = NULL, link = "identity") { 
+ziP <- function (theta = NULL, link = "identity",b=0) { 
 ## zero inflated Poisson parameterized in terms of the log Poisson parameter, gamma. 
 ## eta = theta[1] + exp(theta[2])*gamma), and 1-p = exp(-exp(eta)) where p is 
 ## probability of presence.
@@ -1471,12 +1475,17 @@ ziP <- function (theta = NULL, link = "identity") {
       iniTheta <-  c(theta[1],theta[2])
       n.theta <- 0 ## no thetas to estimate
   } else iniTheta <- c(0,0) ## inital theta value - start at Poisson
-
-  env <- new.env(parent = environment(ziP))# new.env(parent = .GlobalEnv)
+  
+  env <- new.env(parent = environment(ziP))# new.env(parent = .GlobalEnv) 
+  
+  if (b<0) b <- 0; assign(".b", b, envir = env)
   assign(".Theta", iniTheta, envir = env)
   getTheta <- function(trans=FALSE) { 
   ## trans transforms to the original scale...
     th <- get(".Theta")
+    if (trans) {
+      th[2] <- get(".b") + exp(th[2])
+    }
     th
   }
 
@@ -1487,7 +1496,8 @@ ziP <- function (theta = NULL, link = "identity") {
   dev.resids <- function(y, mu, wt,theta=NULL) {
     ## this version ignores saturated likelihood
     if (is.null(theta)) theta <- get(".Theta")
-    p <- theta[1] + exp(theta[2]) * mu ## l.p. for prob present
+    b <- get(".b")
+    p <- theta[1] + (b + exp(theta[2])) * mu ## l.p. for prob present
     -2*zipll(y,mu,p,deriv=0)$l
   }
   
@@ -1498,7 +1508,8 @@ ziP <- function (theta = NULL, link = "identity") {
     ## with any 2 parameter mapping of lp of mean to lp of prob presence.
     if (is.null(theta)) theta <- get(".Theta")
     deriv <- 1; if (level==1) deriv <- 2 else if (level>1) deriv <- 4 
-    g <- lind(mu,theta,level) ## the derviatives of the transform mapping mu to p
+    b <- get(".b")
+    g <- lind(mu,theta,level,b) ## the derviatives of the transform mapping mu to p
     z <- zipll(y,mu,g$p,deriv)
     oo <- list();n <- length(y)
     if (is.null(wt)) wt <- rep(1,n)
@@ -1513,7 +1524,7 @@ ziP <- function (theta = NULL, link = "identity") {
       oo$Dmuth <- -2*wt*(z$l2[,2]*g$p.th + z$l2[,3]*g$p.l*g$p.th + z$l1[,2]*g$p.lth) 
       oo$Dmu2th <- -2*wt*(z$l3[,2]*g$p.th + 2*z$l3[,3]*g$p.l*g$p.th + 2* z$l2[,2]*g$p.lth + 
        z$l3[,4]*g$p.l^2*g$p.th + z$l2[,3]*(2*g$p.l*g$p.lth + g$p.th*g$p.ll) + z$l1[,2]*g$p.llth)
-      oo$Dmu3 <- -2*wt*(z$l3[,1] + 3*z$l3[,2]*g$p.l + 2*z$l3[,3]*g$p.l^2 + 3*z$l2[,2]*g$p.ll + z$l3[,3]*g$p.l^2 +
+      oo$Dmu3 <- -2*wt*(z$l3[,1] + 3*z$l3[,2]*g$p.l + 3*z$l3[,3]*g$p.l^2 + 3*z$l2[,2]*g$p.ll +
        z$l3[,4]*g$p.l^3 +3*z$l2[,3]*g$p.l*g$p.ll + z$l1[,2]*g$p.lll)
     } 
     if (level>1) {
@@ -1540,6 +1551,7 @@ ziP <- function (theta = NULL, link = "identity") {
         z$l3[,3]*(6*g$p.lth*g$p.l + 3*g$p.th*g$p.ll) + 3*z$l2[,2]*g$p.llth + z$l4[,4]*g$p.th*g$p.l^2 + 
         z$l4[,5]*g$p.th*g$p.l^3 + 3*z$l3[,4]*(g$p.l^2*g$p.lth + g$p.th*g$p.l*g$p.ll) +
         z$l2[,3]*(3*g$p.lth*g$p.ll + 3*g$p.l*g$p.llth + g$p.th*g$p.lll) + z$l1[,2]*g$p.lllth)
+
       oo$Dmu4 <- -2*wt*(z$l4[,1] + 4*z$l4[,2]*g$p.l + 6*z$l4[,3]*g$p.l^2 + 6*z$l3[,2]*g$p.ll + 
         4*z$l4[,4]*g$p.l^3 + 12*z$l3[,3]*g$p.l*g$p.ll + 4*z$l2[,2]*g$p.lll + z$l4[,5] * g$p.l^4 +
         6*z$l3[,4]*g$p.l^2*g$p.ll + z$l2[,3] *(4*g$p.l*g$p.lll + 3*g$p.ll^2) + z$l1[,2]*g$p.llll)
@@ -1550,7 +1562,8 @@ ziP <- function (theta = NULL, link = "identity") {
   
   aic <- function(y, mu, theta=NULL, wt, dev) {
     if (is.null(theta)) theta <- get(".Theta")
-    p <- theta[1] + exp(theta[2]) * mu ## l.p. for prob present
+    b <- get(".b")
+    p <- theta[1] + (b+ exp(theta[2])) * mu ## l.p. for prob present
     sum(-2*wt*zipll(y,mu,p,0)$l)
   }
 
@@ -1613,13 +1626,22 @@ ziP <- function (theta = NULL, link = "identity") {
       rzip <- function(gamma,theta) { ## generate ziP deviates according to model and lp gamma
         y <- gamma; n <- length(y)
         lambda <- exp(gamma)
-        eta <- theta[1] + exp(theta[2])*gamma
+        mlam <- max(c(lambda[is.finite(lambda)],.Machine$double.eps^.2))
+        lambda[!is.finite(lambda)] <- mlam
+        b <- get(".b")
+        eta <- theta[1] + (b+exp(theta[2]))*gamma
         p <- 1- exp(-exp(eta))
         ind <- p > runif(n)
         y[!ind] <- 0
-        np <- sum(ind)
+        #np <- sum(ind)
         ## generate from zero truncated Poisson, given presence...
-        y[ind] <- qpois(runif(np,dpois(0,lambda[ind]),1),lambda[ind])
+        lami <- lambda[ind]
+        yi <- p0 <- dpois(0,lami)
+        nearly1 <- 1 - .Machine$double.eps*10
+        ii <- p0 > nearly1 
+        yi[ii] <- 1 ## lambda so low that almost certainly y=1
+        yi[!ii] <- qpois(runif(sum(!ii),p0[!ii],nearly1),lami[!ii])
+        y[ind] <- yi 
         y
       } 
       rzip(mu,get(".Theta"))
@@ -1696,7 +1718,8 @@ ziP <- function (theta = NULL, link = "identity") {
       se <- if (se) drop(sqrt(pmax(0,rowSums((X%*%Vb)*X)))) else NULL ## se of lin pred
     } else { se <- NULL; gamma <- eta}
     ## now compute linear predictor for probability of presence...
-    eta <- theta[1] + exp(theta[2])*gamma
+    b <- get(".b")
+    eta <- theta[1] + (b+exp(theta[2]))*gamma
     et <- exp(eta)
     mu <- p <- 1 - exp(-et)
     fv <- lambda <- exp(gamma)  
@@ -1718,8 +1741,8 @@ ziP <- function (theta = NULL, link = "identity") {
 
 
    
-  environment(saturated.ll) <- environment(dev.resids) <- 
-  environment(aic) <- environment(getTheta) <- environment(rd) <-
+  environment(saturated.ll) <- environment(dev.resids) <- environment(Dd) <-
+  environment(aic) <- environment(getTheta) <- environment(rd) <- environment(predict) <-
   environment(putTheta) <- env
 
   structure(list(family = "zero inflated Poisson", link = linktemp, linkfun = stats$linkfun,
