@@ -114,7 +114,7 @@ void kd_read(kdtree_type *kd,int *idat,double *ddat) {
   kd->rind = idat + 3 + n;
   kd->huge = *ddat;ddat++;
   /* Now make an array of boxes (all cleared to zero)... */
-  kd->box = (box_type *)R_chk_calloc((size_t)nb,sizeof(box_type));
+  kd->box = (box_type *)CALLOC((size_t)nb,sizeof(box_type));
   /* now work through boxes loading contents */
   pp = idat + 3 + 2*n; /* parents */
   pc1 = pp + nb; /* child1 */
@@ -137,7 +137,7 @@ void kd_read(kdtree_type *kd,int *idat,double *ddat) {
 void kd_sanity(kdtree_type kd) {
   int ok=1,i,*count,n=0;
   for (i=0;i<kd.n_box;i++) if (kd.box[i].p1>n) n = kd.box[i].p1;
-  count = (int *)R_chk_calloc((size_t)n,sizeof(int));
+  count = (int *)CALLOC((size_t)n,sizeof(int));
   for (i=0;i<kd.n_box;i++) if (!kd.box[i].child1) { /* terminal node */
     if (kd.box[i].p1-kd.box[i].p0>1) { Rprintf("More than 2 points in a box!!\n");ok=0;}
     count[kd.box[i].p0]++;
@@ -147,7 +147,7 @@ void kd_sanity(kdtree_type kd) {
     if (count[i]!=1) { Rprintf("point %d in %d boxes!\n",i,count[i]);ok=0;}
   }
   if (ok) Rprintf("kd tree sanity checks\n");
-  R_chk_free(count);
+  FREE(count);
 }
 
 void k_order(int *k,int *ind,double *x,int *n) {
@@ -236,11 +236,11 @@ void k_order(int *k,int *ind,double *x,int *n) {
 
 void free_kdtree(kdtree_type kd) {
 /* free a kdtree. Only use for tree created entirely from complied code,
-   not one read from R. For R only versions R_chk_free(kd.box) is all 
+   not one read from R. For R only versions FREE(kd.box) is all 
    that is needed, as rest uses memory sent in from R.*/
-  R_chk_free(kd.ind);R_chk_free(kd.rind);
-  R_chk_free(kd.box[0].lo); /* storage for box coordinates */
-  R_chk_free(kd.box);
+  FREE(kd.ind);FREE(kd.rind);
+  FREE(kd.box[0].lo); /* storage for box coordinates */
+  FREE(kd.box);
 }
 
 void kd_tree(double *X,int *n, int *d,kdtree_type *kd) {
@@ -258,16 +258,16 @@ void kd_tree(double *X,int *n, int *d,kdtree_type *kd) {
   box_type *box;
   double huge=1e100,*pd,*x,*dum1,*dum2,*dum3;
   /* create index for points... */
-  ind = (int *)R_chk_calloc((size_t) *n,sizeof(int)); 
+  ind = (int *)CALLOC((size_t) *n,sizeof(int)); 
   for (i=0,p=ind;i < *n;i++,p++) *p = i; 
   /* Find the number of boxes in the tree */
   m=2;while (m < *n) m *= 2;
   nb = *n * 2 - m / 2 - 1;
   if (nb > m-1) nb = m - 1; 
   /* Now make an array of boxes (all cleared to zero)... */
-  box = (box_type *)R_chk_calloc((size_t)nb,sizeof(box_type));
+  box = (box_type *)CALLOC((size_t)nb,sizeof(box_type));
   /* allocate storage for box defining coordinates... */ 
-  pd = (double *)R_chk_calloc((size_t)nb * (2 * *d),sizeof(double));
+  pd = (double *)CALLOC((size_t)nb * (2 * *d),sizeof(double));
   for (i=0;i<nb;i++) {
     box[i].lo = pd;pd += *d;
     box[i].hi = pd;pd += *d;
@@ -339,7 +339,7 @@ void kd_tree(double *X,int *n, int *d,kdtree_type *kd) {
     }
   }  
   if (bi!=nb-1) Rprintf("bi not equal to nb-1 %d %d\n",bi,nb-1);
-  rind = (int *)R_chk_calloc((size_t) *n,sizeof(int));
+  rind = (int *)CALLOC((size_t) *n,sizeof(int));
   /* now create index of where ith row of X is in ind */
   for (i=0;i<*n;i++) rind[ind[i]]=i; 
   /* now put tree into kd object */
@@ -568,12 +568,12 @@ void p_area(double *a,double *X,kdtree_type kd,int n,int d) {
   double *wa,*lo,*hi,*x0,*x1,min_w,x;
   int np,bi,i,j,k,ok=1,*count,check;
 
-  wa = (double *)R_chk_calloc((size_t)d,sizeof(double));
-  lo = (double *)R_chk_calloc((size_t)d,sizeof(double));
-  hi = (double *)R_chk_calloc((size_t)d,sizeof(double));
-  x0 = (double *)R_chk_calloc((size_t)d,sizeof(double));
-  x1 = (double *)R_chk_calloc((size_t)d,sizeof(double));
-  count = (int *)R_chk_calloc((size_t)d,sizeof(int));
+  wa = (double *)CALLOC((size_t)d,sizeof(double));
+  lo = (double *)CALLOC((size_t)d,sizeof(double));
+  hi = (double *)CALLOC((size_t)d,sizeof(double));
+  x0 = (double *)CALLOC((size_t)d,sizeof(double));
+  x1 = (double *)CALLOC((size_t)d,sizeof(double));
+  count = (int *)CALLOC((size_t)d,sizeof(int));
 
   /* get average box widths, for fallback purposes */
   for (bi=0;bi<kd.n_box;bi++) {
@@ -644,8 +644,8 @@ void p_area(double *a,double *X,kdtree_type kd,int n,int d) {
     x /= np; /* share it out */
     a[i] = x; /* store it */ 
   }
-  R_chk_free(count);
-  R_chk_free(x0);R_chk_free(x1);R_chk_free(lo);R_chk_free(hi);R_chk_free(wa);
+  FREE(count);
+  FREE(x0);FREE(x1);FREE(lo);FREE(hi);FREE(wa);
 } /* p_area */
 
 
@@ -715,15 +715,15 @@ void Rkradius(double *r,int *idat,double *ddat,double *X,double *x,int *m,int *o
   int d,i,j,n_buff=0,nlist,*list;
   if (*op) { /* output saved nei data */
     for (i=0;i<nn;i++) ni[i]=nei[i];
-    R_chk_free(nei);nn=0;
+    FREE(nei);nn=0;
     return;
   }
   kd_read(&kd,idat,ddat); /* unpack kd tree */
   d = kd.d; /* dimension */
   /* get the r-radius neighbour information... */
-  list = (int *)R_chk_calloc((size_t)kd.n,sizeof(int)); /* list of neighbours of ith point */
+  list = (int *)CALLOC((size_t)kd.n,sizeof(int)); /* list of neighbours of ith point */
   n_buff = kd.n*10;
-  nei = (int *)R_chk_calloc((size_t)n_buff,sizeof(int)); /* global list of neighbours */
+  nei = (int *)CALLOC((size_t)n_buff,sizeof(int)); /* global list of neighbours */
   xx=x;nn=0;off[0]=0;
   for (i=0;i<*m;i++) { /* work through points in x */
     k_radius(*r, kd, X,xx,list,&nlist);
@@ -736,8 +736,8 @@ void Rkradius(double *r,int *idat,double *ddat,double *X,double *x,int *m,int *o
     off[i+1] = nn;
     xx += d; /* next point */
   }
-  R_chk_free(list);
-  R_chk_free(kd.box); /* free storage created by kd_read */
+  FREE(list);
+  FREE(kd.box); /* free storage created by kd_read */
 }
 
 void k_newn_work(double *Xm,kdtree_type kd,double *X,double *dist,int *ni,int*m,int *n,int *d,int *k) {
@@ -760,9 +760,9 @@ void k_newn_work(double *Xm,kdtree_type kd,double *X,double *dist,int *ni,int*m,
   ind = kd.ind;
   box = kd.box;
 
-  dk = (double *)R_chk_calloc((size_t)*k,sizeof(double)); /* distance k-array */
-  ik = (int *)R_chk_calloc((size_t)*k,sizeof(int)); /* corresponding index array */
-  x = (double *)R_chk_calloc((size_t)*d,sizeof(double)); /* array for current point */    
+  dk = (double *)CALLOC((size_t)*k,sizeof(double)); /* distance k-array */
+  ik = (int *)CALLOC((size_t)*k,sizeof(int)); /* corresponding index array */
+  x = (double *)CALLOC((size_t)*d,sizeof(double)); /* array for current point */    
   pcount=0;
 
   for (i=0;i < *m;i++) { /* work through all the points in Xm */
@@ -833,9 +833,9 @@ void k_newn_work(double *Xm,kdtree_type kd,double *X,double *dist,int *ni,int*m,
     }     
   } /* end of points loop (i) */
  
-  R_chk_free(dk);
-  R_chk_free(ik);
-  R_chk_free(x);
+  FREE(dk);
+  FREE(ik);
+  FREE(x);
   *n = pcount;
 } /* k_newn_work */
 
@@ -852,7 +852,7 @@ void Rkdnearest(double *X,int *idat,double *ddat,int *n,double *x, int *m, int *
   d = kd.d; /* dimension */
   /* get the nearest neighbour information... */
   k_newn_work(x,kd,X,dist,ni,m,n,&d,k);
-  R_chk_free(kd.box); /* free storage created by kd_read */
+  FREE(kd.box); /* free storage created by kd_read */
 }
 
 
@@ -867,9 +867,9 @@ void k_nn_work(kdtree_type kd,double *X,double *dist,int *ni,int *n,int *d,int *
   ind = kd.ind;
   box = kd.box;
 
-  dk = (double *)R_chk_calloc((size_t)*k,sizeof(double)); /* distance k-array */
-  ik = (int *)R_chk_calloc((size_t)*k,sizeof(int)); /* corresponding index array */
-  x = (double *)R_chk_calloc((size_t)*d,sizeof(double)); /* array for current point */    
+  dk = (double *)CALLOC((size_t)*k,sizeof(double)); /* distance k-array */
+  ik = (int *)CALLOC((size_t)*k,sizeof(int)); /* corresponding index array */
+  x = (double *)CALLOC((size_t)*d,sizeof(double)); /* array for current point */    
   pcount=0;
 
   for (i=0;i < *n;i++) { /* work through all the points in X */
@@ -950,9 +950,9 @@ void k_nn_work(kdtree_type kd,double *X,double *dist,int *ni,int *n,int *d,int *
     }     
   } /* end of points loop (i) */
  
-  R_chk_free(dk);
-  R_chk_free(ik);
-  R_chk_free(x);
+  FREE(dk);
+  FREE(ik);
+  FREE(x);
   *n = pcount;
 } /* k_nn_work */
 
@@ -1032,10 +1032,10 @@ void kba_nn(double *X,double *dist,double *a,int *ni,int *n,int *d,int *k,
   /* d0 = average of distance to 2d+k nearest neighbours - a useful basic length scale */
   for (d0=0.0,p=dist,p1=dist+ *n * d2k;p<p1;p++) d0 += *p;d0 /= *n * d2k;
 
-  x = (double *)R_chk_calloc((size_t) *d,sizeof(double));
+  x = (double *)CALLOC((size_t) *d,sizeof(double));
   /* need to get typical box scale */ 
-  db = (double *)R_chk_calloc((size_t)*d,sizeof(double));
-  count = (int *)R_chk_calloc((size_t)*d,sizeof(int));
+  db = (double *)CALLOC((size_t)*d,sizeof(double));
+  count = (int *)CALLOC((size_t)*d,sizeof(int));
   for (bi=0;bi<kd.n_box;bi++) {
     for (j=0;j<*d;j++) 
     if (kd.box[bi].lo[j] > -kd.huge&&kd.box[bi].hi[j] < kd.huge) {
@@ -1194,7 +1194,7 @@ void kba_nn(double *X,double *dist,double *a,int *ni,int *n,int *d,int *k,
        if (ni[ii]<0) ni[ii] = -ni[ii] - 1; 
     }
   }
-  R_chk_free(x); free_kdtree(kd);R_chk_free(db);R_chk_free(count);
+  FREE(x); free_kdtree(kd);FREE(db);FREE(count);
 }
 
 
@@ -1214,7 +1214,7 @@ void tri2nei(int *t,int *nt,int *n,int *d,int *off) {
   /* now turn off into an intial version of the final off vector */
   for (i=1;i< *n ;i++) off[i] += off[i-1]; 
   /* create oversized storage for neighbour lists */ 
-  nn = (int *)R_chk_calloc((size_t)off[*n-1],sizeof(int));
+  nn = (int *)CALLOC((size_t)off[*n-1],sizeof(int));
   for (p=nn,p1 = nn + off[*n-1];p<p1;p++) *p = -1; /* -1 codes unused space */
   /* now work through triangles, adding vertices to relevant neighbour lists */
   for (i=0;i<*nt;i++) { /* triangle loop */
@@ -1246,7 +1246,7 @@ void tri2nei(int *t,int *nt,int *n,int *d,int *off) {
     k0 = k1;
   }
   /* so now neighbour lists are stored in t and indexed by off */
-  R_chk_free(nn);
+  FREE(nn);
 } /* end of tri2nei */
 
 void ni_dist_filter(double *X,int *n,int *d,int *ni,int *off,double *mult) {
@@ -1257,7 +1257,7 @@ void ni_dist_filter(double *X,int *n,int *d,int *ni,int *off,double *mult) {
 */
   int i,j,k,i0,i1;
   double *dist,z,z2,md=0.0;
-  dist = (double *)R_chk_calloc((size_t) off[*n-1],sizeof(double)); /* interpoint distances */
+  dist = (double *)CALLOC((size_t) off[*n-1],sizeof(double)); /* interpoint distances */
 
   /* now find the average distance to neighbours */
   i0 = 0;
@@ -1289,7 +1289,7 @@ void ni_dist_filter(double *X,int *n,int *d,int *ni,int *off,double *mult) {
     off[j]=k; /* reset off[i] to how far we actually got */
     i0 = i1;
   }
-  R_chk_free(dist);
+  FREE(dist);
 }
 
 void nei_penalty(double *X,int *n,int *d,double *D,int *ni,int *ii,int *off,
@@ -1336,10 +1336,10 @@ void nei_penalty(double *X,int *n,int *d,double *D,int *ni,int *ii,int *off,
   max_nn++; /* self! */
   if (max_nn<6) max_nn=6;
 
-  M = (double *)R_chk_calloc((size_t) 6 * max_nn,sizeof(double));
-  Mi = (double *)R_chk_calloc((size_t) 6 * max_nn,sizeof(double)); 
-  Vt = (double *)R_chk_calloc((size_t) 6 * 6,sizeof(double));
-  sv = (double *)R_chk_calloc((size_t) 6,sizeof(double));
+  M = (double *)CALLOC((size_t) 6 * max_nn,sizeof(double));
+  Mi = (double *)CALLOC((size_t) 6 * max_nn,sizeof(double)); 
+  Vt = (double *)CALLOC((size_t) 6 * 6,sizeof(double));
+  sv = (double *)CALLOC((size_t) 6,sizeof(double));
 
   /*  Rprintf("Starting main loop...\n");*/
   di = i0 = 0;
@@ -1418,10 +1418,10 @@ void nei_penalty(double *X,int *n,int *d,double *D,int *ni,int *ii,int *off,
     i0=i1;
   }
   /* free memory... */ 
-  R_chk_free(M);
-  R_chk_free(Mi);
-  R_chk_free(Vt);
-  R_chk_free(sv);
+  FREE(M);
+  FREE(Mi);
+  FREE(Vt);
+  FREE(sv);
 } /* end of tri_penalty */
 
 
@@ -1462,9 +1462,9 @@ void ss_setup(double *ub,double *lb,double *x,double *w, int *n) {
    set up the lower band coming from the Choleski decomposition of H (3.1)*/
   double *h,*hh,*hh1,*lb1,*ub2,*ub1;
   int i;
-  h = (double *)R_chk_calloc((size_t) *n,sizeof(double));
-  hh=(double *)R_chk_calloc((size_t) *n,sizeof(double));
-  hh1=(double *)R_chk_calloc((size_t) *n,sizeof(double));
+  h = (double *)CALLOC((size_t) *n,sizeof(double));
+  hh=(double *)CALLOC((size_t) *n,sizeof(double));
+  hh1=(double *)CALLOC((size_t) *n,sizeof(double));
   for (i=0;i< *n-1;i++) h[i]=x[i+1]-x[i];
   for (i=0;i< *n-2;i++) hh[i]=2.0*(h[i]+h[i+1])/3.0;
   for (i=0;i< *n-3;i++) hh1[i]=h[i+1]/3.0;
@@ -1483,7 +1483,7 @@ void ss_setup(double *ub,double *lb,double *x,double *w, int *n) {
     ub1[i] = -w[i+1]*(1/h[i]+1/h[i+1]);
     ub2[i] = w[i+2]/h[i+1];
   }
-  R_chk_free(h);R_chk_free(hh);R_chk_free(hh1);
+  FREE(h);FREE(hh);FREE(hh1);
 }
 
 
@@ -1533,7 +1533,7 @@ void sspl_construct(double *lambda,double *x,double *w,double *U,double *V,
 
   for (i=0;i<*n;i++) w[i] = 1/w[i];  /* convert to H and de H convention */
 
-  ub = (double *)R_chk_calloc((size_t)(*n * 3),sizeof(double));
+  ub = (double *)CALLOC((size_t)(*n * 3),sizeof(double));
   ss_setup(ub,lb,x,w,n);
   rho = sqrt(*lambda);
   /* multiply the upper band of the matrix by the smoothing parameter */
@@ -1660,7 +1660,7 @@ void sspl_construct(double *lambda,double *x,double *w,double *U,double *V,
   diagA[1]=L22*L22+L12*L12;
   diagA[0]=L11*L11;
   for (i=0;i<*n;i++) diagA[i] = 1.0 - diagA[i];
-  R_chk_free(ub);
+  FREE(ub);
 }
 
 
@@ -1677,7 +1677,7 @@ void sspl_apply(double *y,double *x,double *w,double *U,double *V,int *n,int *nf
   double *Wy,*U0s,*U0c,*U1s,*U1c,
     *V0s,*V0c,*V1s,*V1c,*p,*p1,*p2,w2,*xx;
   if (*nf > *n) { /* deal with duplicates */
-   xx = (double *)R_chk_calloc((size_t)*nf,sizeof(double));
+   xx = (double *)CALLOC((size_t)*nf,sizeof(double));
    for (p=x,p1=x + *nf,p2=xx;p<p1;p++,p2++) *p2 = *p;
    k=0;ok=1;
    for (i=1;i<*nf;i++) if (xx[k] + *tol < xx[i]) { 
@@ -1692,14 +1692,14 @@ void sspl_apply(double *y,double *x,double *w,double *U,double *V,int *n,int *nf
      ok=0;
    }
    if (!ok) { w[k] = sqrt(w2);y[k]/=w2;}
-   R_chk_free(xx);
+   FREE(xx);
   }
 
   for (i=0;i<*n;i++) w[i] = 1/w[i];  /* convert to H and de H convention */
 
   /* ... y now contains weighted averages at unique x values, w corresponding weights */
   
-  Wy = (double *)R_chk_calloc((size_t) 2 * *n,sizeof(double));
+  Wy = (double *)CALLOC((size_t) 2 * *n,sizeof(double));
   for (i=0;i < *n;i++) Wy[i] = y[i]/w[i];
   /* set up pointers to various rotation pairs.. */
   U0s = U;U0c = U + *n;
@@ -1747,7 +1747,7 @@ void sspl_apply(double *y,double *x,double *w,double *U,double *V,int *n,int *nf
   } else {
     for (i=0;i<*n;i++) y[i] = Wy[i];
   }
-  R_chk_free(Wy);
+  FREE(Wy);
 }
 
 void sspl_mapply(double *y,double *x,double *w,double *U,double *V,int *n,int *nf,double *tol,int *m) {
@@ -1756,8 +1756,8 @@ void sspl_mapply(double *y,double *x,double *w,double *U,double *V,int *n,int *n
   double *xx,*ww,*p,*p1,*p2;
   if (*m > 1 && *nf != *n) xw_store=1;
   if (xw_store) { /* must store original x and w */
-    xx = (double *)R_chk_calloc((size_t)*nf,sizeof(double));
-    ww = (double *)R_chk_calloc((size_t)*nf,sizeof(double));
+    xx = (double *)CALLOC((size_t)*nf,sizeof(double));
+    ww = (double *)CALLOC((size_t)*nf,sizeof(double));
     for (p=xx,p1=xx + *nf,p2=x;p<p1;p++,p2++) *p = *p2;
     for (p=ww,p1=ww + *nf,p2=w;p<p1;p++,p2++) *p = *p2;
   } 
@@ -1769,7 +1769,7 @@ void sspl_mapply(double *y,double *x,double *w,double *U,double *V,int *n,int *n
     sspl_apply(y,x,w,U,V,n,nf,tol); /* smooth y */
     y += *nf;
   }
-  if (xw_store) {R_chk_free(xx);R_chk_free(ww);}
+  if (xw_store) {FREE(xx);FREE(ww);}
 }
 
 void ss_coeffs(double *lb,double *a,double *b,double *c,double *d,double *x, int *n) { 
@@ -1782,9 +1782,9 @@ void ss_coeffs(double *lb,double *a,double *b,double *c,double *d,double *x, int
  */
   double *GTA,*z,*h,*lb1;
   int i;
-  GTA=(double *)R_chk_calloc((size_t)*n,sizeof(double));
-  z=(double *)R_chk_calloc((size_t)*n,sizeof(double));
-  h=(double *)R_chk_calloc((size_t)*n-1,sizeof(double));
+  GTA=(double *)CALLOC((size_t)*n,sizeof(double));
+  z=(double *)CALLOC((size_t)*n,sizeof(double));
+  h=(double *)CALLOC((size_t)*n-1,sizeof(double));
   for (i=0;i<*n-1;i++) h[i]=x[i+1]-x[i];
   for (i=0;i<*n-2;i++)
   GTA[i]=a[i]/h[i]-a[i+1]*(1/h[i]+1/h[i+1])+a[i+2]/h[i+1];
@@ -1799,7 +1799,7 @@ void ss_coeffs(double *lb,double *a,double *b,double *c,double *d,double *x, int
   { d[i]=(c[i+1]-c[i])/(3*h[i]);
     b[i]=(a[i+1]-a[i])/h[i]-c[i]*h[i]-d[i]*h[i]*h[i];
   }
-  R_chk_free(GTA);R_chk_free(z);R_chk_free(h);
+  FREE(GTA);FREE(z);FREE(h);
 }
 
 
